// src/auth.rs
use actix_web::error::{ErrorInternalServerError, ErrorUnauthorized}; // Specific error types
use actix_web::{
    dev::Payload, http::header::AUTHORIZATION, web, Error as ActixWebError, FromRequest,
    HttpRequest, Result as ActixResult,
};
use chrono::{Duration, Utc}; // Import chrono for time checks
use futures::future::{ready, Ready};
use log; // Use the log crate
use rusqlite::Connection;
use std::sync::{Arc, Mutex};

// Represents an authenticated user via token
pub struct Auth {
    pub user_id: String,
}

impl FromRequest for Auth {
    // Use actix_web::Error for consistency in error handling within Actix
    type Error = ActixWebError;
    // Use Ready from futures 0.3
    type Future = Ready<Result<Self, Self::Error>>;

    fn from_request(req: &HttpRequest, _: &mut Payload) -> Self::Future {
        // Extract database connection pool from application data
        // Replace .expect() with proper error handling
        let db_data_result = req.app_data::<web::Data<Arc<Mutex<Connection>>>>();

        let db_data = match db_data_result {
            Some(data) => data,
            None => {
                log::error!("Database connection missing in application data configuration.");
                return ready(Err(ErrorInternalServerError(
                    "Internal server error (app configuration)",
                )));
            }
        };

        // Extract Authorization header
        let auth_header = req.headers().get(AUTHORIZATION);

        if let Some(auth_header_value) = auth_header {
            // Convert header value to string
            if let Ok(auth_str) = auth_header_value.to_str() {
                // Check if it starts with "Bearer "
                if auth_str.starts_with("Bearer ") {
                    // Extract the token part
                    let token = &auth_str[7..];

                    // Lock the mutex to get access to the connection
                    // Handle potential mutex poisoning explicitly
                    let conn_guard = match db_data.lock() {
                        Ok(guard) => guard,
                        Err(poisoned) => {
                            log::error!("Database mutex poisoned: {}", poisoned);
                            // Return internal server error if mutex is poisoned
                            return ready(Err(ErrorInternalServerError(
                                "Internal server error (database lock)",
                            )));
                        }
                    };

                    // Validate the token against the database (now includes expiration check)
                    match super::db::validate_token(&conn_guard, token) {
                        // Token is valid and not expired, return Ok with Auth struct
                        Ok(Some(user_id)) => {
                            log::debug!("Token validated successfully for user_id: {}", user_id);
                            ready(Ok(Auth { user_id }))
                        }
                        // Token is invalid, not found, or expired
                        Ok(None) => {
                            log::warn!("Invalid or expired token received"); // Avoid logging token
                            ready(Err(ErrorUnauthorized("Invalid or expired token")))
                        }
                        // Database error during token validation
                        Err(e) => {
                            log::error!("Database error during token validation: {:?}", e);
                            // Return Unauthorized to avoid leaking internal error details
                            // Consider mapping specific DB errors if needed, but Unauthorized is generally safe
                            ready(Err(ErrorUnauthorized("Token validation failed")))
                        }
                    }
                } else {
                    // Header present but not "Bearer " format
                    log::warn!("Invalid Authorization header format (not Bearer)");
                    ready(Err(ErrorUnauthorized("Invalid token format")))
                }
            } else {
                // Header value contains invalid characters
                log::warn!("Authorization header contains invalid characters");
                ready(Err(ErrorUnauthorized("Invalid token value")))
            }
        } else {
            // Authorization header is missing
            log::warn!("Missing Authorization header");
            ready(Err(ErrorUnauthorized("Missing authorization token")))
        }
    }
}
