// src/db.rs
use anyhow::{anyhow, Context, Result as AnyhowResult};
use bcrypt::{hash, verify, DEFAULT_COST};
use chrono::{Duration as ChronoDuration, Utc}; // Use Utc for timestamps
use log; // Use the log crate
use rusqlite::{
    params, types::Value as RusqliteValue, Connection, OptionalExtension, Result as RusqliteResult,
};
use std::env;
use uuid::Uuid;

use crate::models;

// Configurable token lifetime (e.g., from environment variable or default)
const TOKEN_LIFETIME_HOURS: i64 = 24; // Default to 24 hours

// Initialize the database connection and create tables if they don't exist
pub fn init_db(database_url: &str) -> AnyhowResult<Connection> {
    log::info!("Attempting to open or create database at: {}", database_url);
    let conn = Connection::open(database_url)
        .context(format!("Failed to open the database at {}", database_url))?;

    log::debug!("Creating 'users' table if not exists...");
    conn.execute(
        "CREATE TABLE IF NOT EXISTS users (
            id TEXT PRIMARY KEY,
            username TEXT NOT NULL UNIQUE,
            password TEXT NOT NULL,             -- Stores bcrypt hashed password
            token TEXT UNIQUE,                  -- Stores the current session token (UUID)
            token_expires_at DATETIME           -- Timestamp when the token expires
        )",
        [],
    )
    .context("Failed to create 'users' table")?;

    log::debug!("Creating 'forms' table if not exists...");
    // Storing complex form definitions as JSON blobs in TEXT columns is pragmatic
    // but sacrifices DB-level type safety and query capabilities. Ensure robust
    // application-level validation and consider backup strategies carefully.
    conn.execute(
        "CREATE TABLE IF NOT EXISTS forms (
            id TEXT PRIMARY KEY,
            name TEXT NOT NULL,
            fields TEXT NOT NULL, -- Stores JSON definition of form fields
            created_at DATETIME DEFAULT CURRENT_TIMESTAMP
        )",
        [],
    )
    .context("Failed to create 'forms' table")?;

    log::debug!("Creating 'submissions' table if not exists...");
    // Storing submission data as JSON blobs has similar tradeoffs as form fields.
    conn.execute(
        "CREATE TABLE IF NOT EXISTS submissions (
            id TEXT PRIMARY KEY,
            form_id TEXT NOT NULL,
            data TEXT NOT NULL,    -- Stores JSON submission data
            created_at DATETIME DEFAULT CURRENT_TIMESTAMP,
            FOREIGN KEY (form_id) REFERENCES forms (id) ON DELETE CASCADE
        )",
        [],
    )
    .context("Failed to create 'submissions' table")?;

    // Setup the initial admin user if it doesn't exist, using environment variables
    setup_initial_admin(&conn).context("Failed to setup initial admin user")?;

    log::info!("Database initialization complete.");
    Ok(conn)
}

// Sets up the initial admin user from *required* environment variables if it doesn't exist
fn setup_initial_admin(conn: &Connection) -> AnyhowResult<()> {
    // CRITICAL SECURITY CHANGE: Remove default credentials. Require env vars.
    let initial_admin_username = env::var("INITIAL_ADMIN_USERNAME")
        .map_err(|_| anyhow!("FATAL: INITIAL_ADMIN_USERNAME environment variable is not set."))?;
    let initial_admin_password = env::var("INITIAL_ADMIN_PASSWORD")
        .map_err(|_| anyhow!("FATAL: INITIAL_ADMIN_PASSWORD environment variable is not set."))?;

    if initial_admin_username.is_empty() || initial_admin_password.is_empty() {
        return Err(anyhow!(
            "FATAL: INITIAL_ADMIN_USERNAME and INITIAL_ADMIN_PASSWORD must not be empty."
        ));
    }

    // Check password complexity? (Optional enhancement)

    add_user_if_not_exists(conn, &initial_admin_username, &initial_admin_password)
        .context("Failed during initial admin user setup")?;
    Ok(())
}

// Adds a user with a hashed password if the username doesn't exist
pub fn add_user_if_not_exists(
    conn: &Connection,
    username: &str,
    password: &str,
) -> AnyhowResult<bool> {
    // Check if user already exists
    let user_exists: bool = conn
        .query_row(
            "SELECT EXISTS(SELECT 1 FROM users WHERE username = ?1)",
            params![username],
            |row| row.get::<_, i32>(0),
        )
        .context(format!("Failed to check existence of user '{}'", username))?
        == 1;

    if user_exists {
        log::debug!("User '{}' already exists, skipping creation.", username);
        return Ok(false); // User already exists, nothing added
    }

    // Generate a UUID for the new user
    let user_id = Uuid::new_v4().to_string();

    // Hash the password using bcrypt
    // Ensure the cost factor is appropriate for your security needs and hardware.
    // Higher cost means slower hashing and verification, but better resistance to brute-force.
    log::debug!(
        "Hashing password for user '{}' with cost {}",
        username,
        DEFAULT_COST
    );
    let hashed_password = hash(password, DEFAULT_COST).context("Failed to hash password")?;

    // Insert the new user (token and expiry are initially NULL)
    log::info!("Creating new user '{}' with ID: {}", username, user_id);
    conn.execute(
        "INSERT INTO users (id, username, password) VALUES (?1, ?2, ?3)",
        params![user_id, username, hashed_password],
    )
    .context(format!("Failed to insert user '{}'", username))?;

    Ok(true) // User was added
}

// Validate a session token and return the associated user ID if valid and not expired
pub fn validate_token(conn: &Connection, token: &str) -> AnyhowResult<Option<String>> {
    log::debug!("Validating received token (existence and expiration)...");
    let mut stmt = conn.prepare(
        // Select user ID only if token matches AND it hasn't expired
        "SELECT id FROM users WHERE token = ?1 AND token_expires_at IS NOT NULL AND token_expires_at > ?2"
    ).context("Failed to prepare query for validating token")?;

    let now_ts = Utc::now().to_rfc3339(); // Use ISO 8601 / RFC 3339 format compatible with SQLite DATETIME

    let user_id_option: Option<String> = stmt
        .query_row(params![token, now_ts], |row| row.get(0))
        .optional() // Makes it return Option instead of erroring on no rows
        .context("Failed to execute query for validating token")?;

    if user_id_option.is_some() {
        log::debug!("Token validation successful.");
    } else {
        // This covers token not found OR token expired
        log::debug!("Token validation failed (token not found or expired).");
    }

    Ok(user_id_option)
}

// Invalidate a user's token (e.g., on logout) by setting it to NULL and clearing expiration
pub fn invalidate_token(conn: &Connection, user_id: &str) -> AnyhowResult<()> {
    log::debug!("Invalidating token for user_id {}", user_id);
    conn.execute(
        "UPDATE users SET token = NULL, token_expires_at = NULL WHERE id = ?1",
        params![user_id],
    )
    .context(format!(
        "Failed to invalidate token for user_id {}",
        user_id
    ))?;
    Ok(())
}

// Authenticate a user by username and password, returning user ID and hash if successful
pub fn authenticate_user(
    conn: &Connection,
    username: &str,
    password: &str,
) -> AnyhowResult<Option<models::UserAuthData>> {
    log::debug!("Attempting to authenticate user: {}", username);
    let mut stmt = conn
        .prepare("SELECT id, password FROM users WHERE username = ?1")
        .context("Failed to prepare query for authenticating user")?;

    let result = stmt
        .query_row(params![username], |row| {
            Ok(models::UserAuthData {
                id: row.get(0)?,
                hashed_password: row.get(1)?,
            })
        })
        .optional()
        .context(format!(
            "Failed to execute query to fetch auth data for user '{}'",
            username
        ))?;

    match result {
        Some(user_data) => {
            // Verify the provided password against the stored hash
            let is_valid = verify(password, &user_data.hashed_password)
                .context("Failed to verify password hash")?;

            if is_valid {
                log::info!("Authentication successful for user: {}", username);
                Ok(Some(user_data)) // Return user ID and hash
            } else {
                log::warn!(
                    "Authentication failed for user '{}' (invalid password)",
                    username
                );
                Ok(None) // Invalid password
            }
        }
        None => {
            log::warn!(
                "Authentication failed for user '{}' (user not found)",
                username
            );
            Ok(None) // User not found
        }
    }
}

// Generate and save a new session token (with expiration) for a user
pub fn generate_and_set_token_for_user(conn: &Connection, user_id: &str) -> AnyhowResult<String> {
    let new_token = Uuid::new_v4().to_string();
    // Calculate expiration time
    let expires_at = Utc::now() + ChronoDuration::hours(TOKEN_LIFETIME_HOURS);
    let expires_at_ts = expires_at.to_rfc3339(); // Store as string

    log::debug!(
        "Generating new token for user_id {} expiring at {}",
        user_id,
        expires_at_ts
    );

    conn.execute(
        "UPDATE users SET token = ?1, token_expires_at = ?2 WHERE id = ?3",
        params![new_token, expires_at_ts, user_id],
    )
    .context(format!("Failed to update token for user_id {}", user_id))?;

    Ok(new_token)
}

// Fetch a specific form definition by its ID
pub fn get_form_definition(conn: &Connection, form_id: &str) -> AnyhowResult<Option<models::Form>> {
    let mut stmt = conn
        .prepare("SELECT id, name, fields FROM forms WHERE id = ?1")
        .context("Failed to prepare statement for getting form definition")?;

    let form_option = stmt
        .query_row(params![form_id], |row| {
            let id: String = row.get(0)?;
            let name: String = row.get(1)?;
            let fields_str: String = row.get(2)?;

            // Ensure fields can be parsed as valid JSON Value
            let fields: serde_json::Value = serde_json::from_str(&fields_str).map_err(|e| {
                // Log clearly that this is a data integrity issue
                log::error!(
                    "Database integrity error: Failed to parse 'fields' JSON for form_id {}: {}. Content: '{}'",
                    id, e, fields_str // Log content if not too large/sensitive
                );
                rusqlite::Error::FromSqlConversionFailure(
                    2,
                    rusqlite::types::Type::Text,
                    Box::new(e),
                )
            })?;

            // **Basic check**: Ensure fields is an array (common pattern for form definitions)
            if !fields.is_array() {
                log::error!(
                    "Database integrity error: 'fields' column for form_id {} is not a JSON array.",
                    id
                );
                return Err(rusqlite::Error::FromSqlConversionFailure(
                    2,
                    rusqlite::types::Type::Text,
                    "Form fields definition is not a valid JSON array".into(),
                ));
            }

            Ok(models::Form {
                id: Some(id),
                name,
                fields,
            })
        })
        .optional() // Handle case where form_id doesn't exist
        .context(format!(
            "Failed to execute query for form definition with id {}",
            form_id
        ))?;

    Ok(form_option)
}
