// src/handlers.rs
use crate::auth::Auth;
use crate::models::{Form, LoginCredentials, LoginResponse, Submission};
use actix_web::{web, Error as ActixWebError, HttpResponse, Responder, Result as ActixResult};
use anyhow::Context; // Import anyhow::Context for error chaining
use log;
use regex::Regex; // For pattern validation
use rusqlite::{params, Connection};
use serde_json::{json, Map, Value as JsonValue}; // Alias for clarity
use std::collections::HashMap;
use std::error::Error as StdError;
use std::sync::{Arc, Mutex};
use uuid::Uuid;

// --- Helper Function for Database Access ---

// Gets a database connection from the request data, handling lock errors consistently.
fn get_db_conn(
    db: &web::Data<Arc<Mutex<Connection>>>,
) -> Result<std::sync::MutexGuard<'_, Connection>, ActixWebError> {
    db.lock().map_err(|poisoned| {
        log::error!("Database mutex poisoned: {}", poisoned);
        actix_web::error::ErrorInternalServerError("Internal database error (mutex lock)")
    })
}

// --- Helper Function for Validation ---

/// Validates submission data against the form field definitions with enhanced checks.
///
/// Expected field definition properties:
/// - `name`: string (required)
/// - `type`: string (e.g., "string", "number", "boolean", "email", "url", "object", "array") (required)
/// - `required`: boolean (optional, default: false)
/// - `maxLength`: number (for "string" type)
/// - `minLength`: number (for "string" type)
/// - `min`: number (for "number" type)
/// - `max`: number (for "number" type)
/// - `pattern`: string (regex for "string", "email", "url" types)
///
/// Returns `Ok(())` if valid, or `Err(JsonValue)` containing validation errors.
fn validate_submission_against_definition(
    submission_data: &JsonValue,
    form_definition_fields: &JsonValue,
) -> Result<(), JsonValue> {
    let mut errors: HashMap<String, String> = HashMap::new();

    // Ensure 'fields' in the definition is a JSON array
    let field_definitions = match form_definition_fields.as_array() {
        Some(defs) => defs,
        None => {
            log::error!(
                "Form definition 'fields' is not a JSON array. Def: {:?}",
                form_definition_fields
            );
            errors.insert(
                "_internal".to_string(),
                "Invalid form definition format (not an array)".to_string(),
            );
            return Err(json!({ "validation_errors": errors }));
        }
    };

    // Ensure the submission data is a JSON object
    let data_map = match submission_data.as_object() {
        Some(map) => map,
        None => {
            errors.insert(
                "_submission".to_string(),
                "Submission data must be a JSON object".to_string(),
            );
            return Err(json!({ "validation_errors": errors }));
        }
    };

    // Build a map of valid field names to their definitions from the definition for quick lookup
    let defined_field_names: HashMap<String, &Map<String, JsonValue>> = field_definitions
        .iter()
        .filter_map(|val| val.as_object())
        .filter_map(|def| {
            def.get("name")
                .and_then(JsonValue::as_str)
                .map(|name| (name.to_string(), def))
        })
        .collect();

    // 1. Check for submitted fields that are NOT in the definition
    for submitted_key in data_map.keys() {
        if !defined_field_names.contains_key(submitted_key) {
            errors.insert(
                submitted_key.clone(),
                "Unexpected field submitted".to_string(),
            );
        }
    }
    // Exit early if unexpected fields were found
    if !errors.is_empty() {
        log::warn!("Submission validation failed: Unexpected fields submitted.");
        return Err(json!({ "validation_errors": errors }));
    }

    // 2. Iterate through each field definition and validate corresponding submitted data
    for (field_name, field_def) in &defined_field_names {
        // Extract properties using helper functions for clarity
        let field_type = field_def
            .get("type")
            .and_then(JsonValue::as_str)
            .unwrap_or("string"); // Default to "string" if type is missing or not a string
        let is_required = field_def
            .get("required")
            .and_then(JsonValue::as_bool)
            .unwrap_or(false); // Default to false if required is missing or not a boolean
        let min_length = field_def.get("minLength").and_then(JsonValue::as_u64);
        let max_length = field_def.get("maxLength").and_then(JsonValue::as_u64);
        let min_value = field_def.get("min").and_then(JsonValue::as_f64); // Use f64 for flexibility
        let max_value = field_def.get("max").and_then(JsonValue::as_f64);
        let pattern = field_def.get("pattern").and_then(JsonValue::as_str);

        match data_map.get(field_name) {
            Some(submitted_value) if !submitted_value.is_null() => {
                // Field is present and not null, perform type and constraint checks
                let mut type_error = None;
                let mut constraint_errors = vec![];

                match field_type {
                    "string" | "email" | "url" => {
                        if let Some(s) = submitted_value.as_str() {
                            if let Some(min) = min_length {
                                if (s.chars().count() as u64) < min {
                                    // Use chars().count() for UTF-8 correctness
                                    constraint_errors
                                        .push(format!("Must be at least {} characters long", min));
                                }
                            }
                            if let Some(max) = max_length {
                                if (s.chars().count() as u64) > max {
                                    constraint_errors.push(format!(
                                        "Must be no more than {} characters long",
                                        max
                                    ));
                                }
                            }
                            if let Some(pat) = pattern {
                                // Consider caching compiled Regex if performance is critical
                                // and patterns are reused frequently across requests.
                                match Regex::new(pat) {
                                    Ok(re) => {
                                        if !re.is_match(s) {
                                           constraint_errors.push(format!("Does not match required pattern"));
                                        }
                                    }
                                    Err(e) => log::warn!("Invalid regex pattern '{}' in form definition for field '{}': {}", pat, field_name, e), // Log regex compilation error
                                }
                            }
                            // Specific checks for email/url
                            if field_type == "email" {
                                // Basic email regex (adjust for stricter needs or use a validation crate)
                                // This regex is very basic and allows many technically invalid addresses.
                                // Consider crates like `validator` for more robust validation.
                                let email_regex =
                                    Regex::new(r"^[^\s@]+@[^\s@]+\.[^\s@]+$").unwrap(); // Safe unwrap for known valid regex
                                if !email_regex.is_match(s) {
                                    constraint_errors
                                        .push("Must be a valid email address".to_string());
                                }
                            }
                            if field_type == "url" {
                                // Basic URL check (consider `url` crate for robustness)
                                if url::Url::parse(s).is_err() {
                                    constraint_errors.push("Must be a valid URL".to_string());
                                }
                            }
                        } else {
                            type_error = Some(format!("Expected a string for '{}'", field_name));
                        }
                    }
                    "number" => {
                        // Use as_f64 for flexibility (handles integers and floats)
                        if let Some(num) = submitted_value.as_f64() {
                            if let Some(min) = min_value {
                                if num < min {
                                    constraint_errors.push(format!("Must be at least {}", min));
                                }
                            }
                            if let Some(max) = max_value {
                                if num > max {
                                    constraint_errors.push(format!("Must be no more than {}", max));
                                }
                            }
                        } else {
                            type_error = Some(format!("Expected a number for '{}'", field_name));
                        }
                    }
                    "boolean" => {
                        if !submitted_value.is_boolean() {
                            type_error = Some(format!(
                                "Expected a boolean (true/false) for '{}'",
                                field_name
                            ));
                        }
                    }
                    "object" => {
                        if !submitted_value.is_object() {
                            type_error =
                                Some(format!("Expected a JSON object for '{}'", field_name));
                        }
                        // TODO: Could add deeper validation for object structure here if needed based on definition
                    }
                    "array" => {
                        if !submitted_value.is_array() {
                            type_error =
                                Some(format!("Expected a JSON array for '{}'", field_name));
                        }
                        // TODO: Could add validation for array elements here if needed based on definition
                    }
                    _ => {
                        // Log unsupported types during development/debugging if necessary
                        log::trace!(
                            "Unsupported field type '{}' encountered during validation for field '{}'. Treating as valid.",
                            field_type,
                            field_name
                        );
                        // Assume valid if type is not specifically handled or unknown
                    }
                }

                // Record errors found for this field
                if let Some(err) = type_error {
                    errors.insert(field_name.clone(), err);
                } else if !constraint_errors.is_empty() {
                    // Combine multiple constraint errors if necessary
                    errors.insert(field_name.clone(), constraint_errors.join("; "));
                }
            } // End check for present and non-null value
            Some(_) => {
                // Value is present but explicitly null (e.g., "fieldName": null)
                if is_required {
                    errors.insert(
                        field_name.clone(),
                        "This field is required and cannot be null".to_string(),
                    );
                }
                // Otherwise, null is considered a valid (empty) value for non-required fields
            }
            None => {
                // Field is missing entirely from the submission object
                if is_required {
                    errors.insert(field_name.clone(), "This field is required".to_string());
                }
                // Missing is valid for non-required fields
            }
        } // End match data_map.get(field_name)
    } // End loop through field definitions

    // Check if any errors were collected
    if errors.is_empty() {
        Ok(()) // Validation passed
    } else {
        log::info!(
            "Submission validation failed with {} error(s).", // Log only the count for brevity
            errors.len()
        );
        // Return a JSON object containing the specific validation errors
        Err(json!({ "validation_errors": errors }))
    }
}

// Helper function to convert anyhow::Error to actix_web::Error
fn anyhow_to_actix_error(e: anyhow::Error) -> ActixWebError {
    actix_web::error::ErrorInternalServerError(e.to_string())
}

// --- Public Handlers ---

// POST /login
pub async fn login(
    db: web::Data<Arc<Mutex<Connection>>>,
    creds: web::Json<LoginCredentials>,
) -> ActixResult<impl Responder> {
    let db_conn = db.clone(); // Clone Arc for use in web::block
    let username = creds.username.clone();
    let password = creds.password.clone();

    // Wrap the blocking database operations in web::block
    let auth_result = web::block(move || {
        let conn = db_conn
            .lock()
            .map_err(|_| anyhow::anyhow!("Database mutex poisoned during login lock"))?;
        crate::db::authenticate_user(&conn, &username, &password)
    })
    .await
    .map_err(|e| {
        log::error!("web::block error during authentication: {:?}", e);
        actix_web::error::ErrorInternalServerError("Authentication process failed (blocking error)")
    })?
    .map_err(anyhow_to_actix_error)?;

    match auth_result {
        Some(user_data) => {
            let db_conn_token = db.clone(); // Clone Arc again for token generation
            let user_id = user_data.id.clone();

            // Generate and store a new token within web::block
            let token = web::block(move || {
                let conn = db_conn_token
                    .lock()
                    .map_err(|_| anyhow::anyhow!("Database mutex poisoned during token lock"))?;
                crate::db::generate_and_set_token_for_user(&conn, &user_id)
            })
            .await
            .map_err(|e| {
                log::error!("web::block error during token generation: {:?}", e);
                actix_web::error::ErrorInternalServerError(
                    "Failed to complete login (token generation blocking error)",
                )
            })?
            .map_err(anyhow_to_actix_error)?;

            log::info!("Login successful for user_id: {}", user_data.id);
            Ok(HttpResponse::Ok().json(LoginResponse { token }))
        }
        None => {
            log::warn!("Login failed for username: {}", creds.username);
            // Return 401 Unauthorized for failed login attempts
            Err(actix_web::error::ErrorUnauthorized(
                "Invalid username or password",
            ))
        }
    }
}

// POST /logout
pub async fn logout(
    db: web::Data<Arc<Mutex<Connection>>>,
    auth: Auth, // Requires authentication (extracts user_id from token)
) -> ActixResult<impl Responder> {
    log::info!("User {} requesting logout", auth.user_id);
    let db_conn = db.clone();
    let user_id = auth.user_id.clone();

    // Invalidate the token in the database within web::block
    web::block(move || {
        let conn = db_conn
            .lock()
            .map_err(|_| anyhow::anyhow!("Database mutex poisoned during logout lock"))?;
        crate::db::invalidate_token(&conn, &user_id)
    })
    .await
    .map_err(|e| {
        let user_id = auth.user_id.clone(); // Clone user_id again after the move
        log::error!(
            "web::block error during logout for user {}: {:?}",
            user_id,
            e
        );
        actix_web::error::ErrorInternalServerError("Logout failed (blocking error)")
    })?
    .map_err(anyhow_to_actix_error)?;

    log::info!("User {} logged out successfully", auth.user_id);
    Ok(HttpResponse::Ok().json(json!({ "message": "Logged out successfully" })))
}

// POST /forms/{form_id}/submissions
pub async fn submit_form(
    db: web::Data<Arc<Mutex<Connection>>>,
    path: web::Path<String>,                  // Extracts form_id from path
    submission_payload: web::Json<JsonValue>, // Expect arbitrary JSON payload
) -> ActixResult<impl Responder> {
    let form_id = path.into_inner();
    let submission_data = submission_payload.into_inner(); // Get the JSON data

    // --- Stage 1: Fetch form definition (Read-only, can use shared lock) ---
    let form_definition = {
        // Acquire lock temporarily for the read operation
        let conn = get_db_conn(&db)?;
        match crate::db::get_form_definition(&conn, &form_id) {
            Ok(Some(form)) => form,
            Ok(None) => {
                log::warn!("Submission attempt for non-existent form_id: {}", form_id);
                return Err(actix_web::error::ErrorNotFound("Form not found"));
            }
            Err(e) => {
                log::error!("Failed to fetch form definition for {}: {:?}", form_id, e);
                return Err(actix_web::error::ErrorInternalServerError(
                    "Could not retrieve form information",
                ));
            }
        }
        // Lock is released here when 'conn' goes out of scope
    };

    // --- Stage 2: Validate submission against definition (CPU-bound, no DB lock needed) ---
    if let Err(validation_errors) =
        validate_submission_against_definition(&submission_data, &form_definition.fields)
    {
        log::warn!(
            "Submission validation failed for form_id {}. Errors: {:?}", // Log actual errors if needed (might be verbose)
            form_id,
            validation_errors
        );
        // Return 400 Bad Request with validation error details
        return Ok(HttpResponse::BadRequest().json(validation_errors));
    }

    // --- Stage 3: Serialize validated data and Insert submission (Write operation, use web::block) ---
    let submission_json = match serde_json::to_string(&submission_data) {
        Ok(json_string) => json_string,
        Err(e) => {
            log::error!(
                "Failed to serialize validated submission data for form {}: {}",
                form_id,
                e
            );
            return Err(actix_web::error::ErrorInternalServerError(
                "Failed to process submission data internally",
            ));
        }
    };

    let db_conn_write = db.clone(); // Clone Arc for the blocking operation
    let form_id_clone = form_id.clone(); // Clone for closure
    let submission_id = Uuid::new_v4().to_string(); // Generate unique ID for the submission
    let submission_id_clone = submission_id.clone(); // Clone for closure

    web::block(move || {
        let conn = db_conn_write.lock().map_err(|_| {
            anyhow::anyhow!("Database mutex poisoned during submission insert lock")
        })?;
        conn.execute(
            "INSERT INTO submissions (id, form_id, data) VALUES (?1, ?2, ?3)",
            params![submission_id_clone, form_id_clone, submission_json],
        )
        .context(format!(
            "Failed to insert submission for form {}",
            form_id_clone
        ))
        .map_err(anyhow::Error::from)
    })
    .await
    .map_err(|e| {
        log::error!(
            "web::block error during submission insertion for form {}: {:?}",
            form_id,
            e
        );
        actix_web::error::ErrorInternalServerError("Failed to save submission (blocking error)")
    })?
    .map_err(anyhow_to_actix_error)?;

    log::info!(
        "Successfully inserted submission {} for form_id {}",
        submission_id,
        form_id
    );
    // Return 200 OK with the new submission ID
    Ok(HttpResponse::Ok().json(json!({ "submission_id": submission_id })))
}

// --- Protected Handlers (Require Auth) ---

// POST /forms
pub async fn create_form(
    db: web::Data<Arc<Mutex<Connection>>>,
    auth: Auth, // Authentication check via Auth extractor
    form_payload: web::Json<Form>,
) -> ActixResult<impl Responder> {
    log::info!(
        "User {} attempting to create form: {}",
        auth.user_id,
        form_payload.name
    );

    let mut form = form_payload.into_inner();
    // Generate a new UUID for the form if not provided (or overwrite if provided)
    let form_id = form.id.unwrap_or_else(|| Uuid::new_v4().to_string());
    form.id = Some(form_id.clone()); // Ensure the form object has the ID

    // Basic structural validation: Ensure 'fields' is a JSON array before serialization/saving
    if !form.fields.is_array() {
        log::error!(
            "User {} attempted to create form '{}' ('{}') where 'fields' is not a JSON array.",
            auth.user_id,
            form.name,
            form_id
        );
        return Err(actix_web::error::ErrorBadRequest(
            "Form 'fields' must be a valid JSON array.",
        ));
    }
    // TODO: Add deeper validation of the 'fields' structure itself if needed
    // e.g., check if each element in 'fields' is an object with 'name' and 'type'.

    // Serialize the fields part to JSON string for DB storage
    let fields_json = match serde_json::to_string(&form.fields) {
        Ok(json_str) => json_str,
        Err(e) => {
            log::error!(
                "Failed to serialize form fields for form '{}' ('{}') by user {}: {}",
                form.name,
                form_id,
                auth.user_id,
                e
            );
            return Err(actix_web::error::ErrorInternalServerError(
                "Failed to process form fields internally",
            ));
        }
    };

    // Clone data needed for the blocking database operation
    let db_conn = db.clone();
    // let form_id = form_id; // Already have it from above
    let form_name = form.name.clone();
    let user_id = auth.user_id.clone(); // For logging inside block if needed

    // Insert the form using web::block for the blocking DB write
    web::block(move || {
        let conn = db_conn
            .lock()
            .map_err(|_| anyhow::anyhow!("Database mutex poisoned during form creation lock"))?;
        conn.execute(
            // Consider adding user_id to the forms table if forms are user-specific
            "INSERT INTO forms (id, name, fields) VALUES (?1, ?2, ?3)",
            params![form_id, form_name, fields_json],
        )
        .context("Failed to insert new form into database")
        .map_err(anyhow::Error::from)
    })
    .await
    .map_err(|e| {
        log::error!(
            "web::block error during form creation by user {}: {:?}",
            auth.user_id,
            e
        );
        actix_web::error::ErrorInternalServerError("Failed to create form (blocking error)")
    })?
    .map_err(anyhow_to_actix_error)?;

    log::info!(
        "Successfully created form '{}' with id {} by user {}",
        form.name,
        form.id.as_ref().unwrap(), // Safe unwrap as we set it
        auth.user_id
    );
    // Return 200 OK with the newly created form object (including its ID)
    Ok(HttpResponse::Ok().json(form))
}

// GET /forms
pub async fn get_forms(
    db: web::Data<Arc<Mutex<Connection>>>,
    auth: Auth, // Requires authentication
) -> ActixResult<impl Responder> {
    log::info!("User {} requesting list of forms", auth.user_id);
    let db_conn = db.clone();
    let user_id = auth.user_id.clone(); // Clone for logging context if needed inside block

    // Wrap DB query in web::block as it might be slow with many forms or complex parsing
    let forms_result = web::block(move || {
        let conn = db_conn
            .lock()
            .map_err(|_| anyhow::anyhow!("Database mutex poisoned during get_forms lock"))?;

        let mut stmt = conn
            .prepare("SELECT id, name, fields FROM forms")
            .context("Failed to prepare statement for getting forms")?;

        let forms_iter = stmt
            .query_map([], |row| {
                let id: String = row.get(0)?;
                let name: String = row.get(1)?;
                let fields_str: String = row.get(2)?;

                // Parse the 'fields' JSON string. If it fails, log the error and skip the row.
                let fields: serde_json::Value = match serde_json::from_str(&fields_str) {
                    Ok(json_value) => json_value,
                    Err(e) => {
                        // Log the data integrity issue clearly
                        log::error!(
                            "DB Parse Error: Failed to parse 'fields' JSON for form id {}: {}. Skipping this form.",
                             id, e
                        );
                        // Return a special error that `filter_map` below can catch,
                        // without failing the entire query_map.
                        // Using a specific rusqlite error type here is okay.
                        return Err(rusqlite::Error::FromSqlConversionFailure(
                            2, // Column index
                            rusqlite::types::Type::Text,
                            Box::new(e) // Box the original error
                        ));
                    }
                };

                Ok(Form { id: Some(id), name, fields })
            })
            .context("Failed to execute query map for getting forms")?;

        // Collect results, filtering out rows that failed parsing WITHIN the block
        let forms: Vec<Form> = forms_iter
            .filter_map(|result| match result {
                Ok(form) => Some(form),
                Err(e) => {
                    // Error was already logged inside the query_map closure.
                    // We just filter out the failed row here.
                    log::warn!("Skipping a form row due to a processing error: {}", e);
                    None // Skip this row
                }
            })
            .collect();

        Ok::<_, anyhow::Error>(forms) // Ensure block returns Result compatible with flattening
    })
    .await
    .map_err(|e| {
        // Handle web::block error
        log::error!("web::block error during get_forms for user {}: {:?}", user_id, e);
        actix_web::error::ErrorInternalServerError("Failed to retrieve forms (blocking error)")
    })?
    .map_err(anyhow_to_actix_error)?; // Flatten Result<Result<Vec<Form>, anyhow::Error>, BlockingError>

    log::debug!(
        "Returning {} forms for user {}",
        forms_result.len(),
        auth.user_id
    );
    Ok(HttpResponse::Ok().json(forms_result))
}

// GET /forms/{form_id}/submissions
pub async fn get_submissions(
    db: web::Data<Arc<Mutex<Connection>>>,
    auth: Auth,              // Requires authentication
    path: web::Path<String>, // Extracts form_id from the path
) -> ActixResult<impl Responder> {
    let form_id = path.into_inner();
    log::info!(
        "User {} requesting submissions for form_id: {}",
        auth.user_id,
        form_id
    );

    let db_conn = db.clone();
    let form_id_clone = form_id.clone();
    let user_id = auth.user_id.clone(); // Clone for logging context

    // Wrap DB queries (existence check + fetching submissions) in web::block
    let submissions_result = web::block(move || {
        let conn = db_conn
            .lock()
            .map_err(|_| anyhow::anyhow!("Database mutex poisoned during get_submissions lock"))?;

        // 1. Check if the form exists first
        let form_exists: bool = match conn.query_row(
            "SELECT EXISTS(SELECT 1 FROM forms WHERE id = ?1 LIMIT 1)", // Added LIMIT 1 for potential optimization
            params![form_id_clone],
            |row| row.get::<_, i32>(0), // sqlite returns 0 or 1 for EXISTS
        ) {
            Ok(count) => count == 1,
            Err(rusqlite::Error::QueryReturnedNoRows) => false, // Should not happen with EXISTS, but handle defensively
            Err(e) => return Err(anyhow::Error::from(e) // Propagate other DB errors
                .context(format!("Failed check existence of form {}", form_id_clone))),
        };

        if !form_exists {
             // Use Ok(None) to signal "form not found" to the calling async context
            return Ok(None);
        }

        // 2. If form exists, fetch its submissions
        let mut stmt = conn.prepare(
                "SELECT id, form_id, data, created_at FROM submissions WHERE form_id = ?1 ORDER BY created_at DESC", // Include created_at if needed
            )
            .context(format!("Failed to prepare statement for getting submissions for form {}", form_id_clone))?;

        let submissions_iter = stmt
            .query_map(params![form_id_clone], |row| {
                let id: String = row.get(0)?;
                let form_id_db: String = row.get(1)?;
                let data_str: String = row.get(2)?;
                // let created_at: String = row.get(3)?; // Example: If you fetch created_at

                // Parse the 'data' JSON string, handling potential errors
                let data: serde_json::Value = match serde_json::from_str(&data_str) {
                    Ok(json_value) => json_value,
                    Err(e) => {
                         log::error!(
                            "DB Parse Error: Failed to parse 'data' JSON for submission_id {}: {}. Skipping.",
                            id, e
                        );
                        // Return specific error for filter_map
                        return Err(rusqlite::Error::FromSqlConversionFailure(
                            2, rusqlite::types::Type::Text, Box::new(e)
                        ));
                    }
                };

                Ok(Submission { id, form_id: form_id_db, data }) // Add created_at if fetched
            })
             .context(format!("Failed to execute query map for getting submissions for form {}", form_id_clone))?;

        // Collect valid submissions, filtering out rows that failed parsing
        let submissions: Vec<Submission> = submissions_iter
            .filter_map(|result| match result {
                Ok(submission) => Some(submission),
                Err(e) => {
                    log::warn!("Skipping a submission row due to processing error: {}", e);
                    None // Skip this row
                }
            })
            .collect();

        Ok(Some(submissions)) // Indicate success with the (potentially empty) list of submissions

    })
    .await
    .map_err(|e| { // Handle web::block error (cancellation, panic)
        log::error!("web::block error during get_submissions for form {} by user {}: {:?}", form_id, user_id, e);
        actix_web::error::ErrorInternalServerError("Failed to retrieve submissions (blocking error)")
    })?
    .map_err(anyhow_to_actix_error)?; // Flatten Result<Result<Option<Vec<Submission>>, anyhow::Error>, BlockingError>

    // Process the result obtained from the web::block
    match submissions_result {
        Some(submissions) => {
            // Form exists, return the found submissions (might be an empty list)
            log::debug!(
                "Returning {} submissions for form {} requested by user {}",
                submissions.len(),
                form_id,
                auth.user_id
            );
            Ok(HttpResponse::Ok().json(submissions))
        }
        None => {
            // Form was not found (signaled by Ok(None) from the block)
            log::warn!(
                "Attempt by user {} to get submissions for non-existent form_id: {}",
                auth.user_id,
                form_id
            );
            Err(actix_web::error::ErrorNotFound("Form not found"))
        }
    }
}
