// src/main.rs
use actix_cors::Cors;
use actix_files as fs;
use actix_web::{http::header, middleware::Logger, web, App, HttpServer}; // Added Logger explicitly
use dotenv::dotenv;
use log;
use std::env;
use std::io::Result as IoResult; // Alias for clarity
use std::process;
use std::sync::{Arc, Mutex};

// Import modules
mod auth;
mod db;
mod handlers;
mod models;

#[actix_web::main]
async fn main() -> IoResult<()> {
    dotenv().ok(); // Load .env file

    // Initialize logger (using RUST_LOG env var)
    env_logger::Builder::from_env(env_logger::Env::default().default_filter_or("info")).init();

    // --- Configuration (Environment Variables) ---
    // CRITICAL: Database URL is required
    let database_url = env::var("DATABASE_URL").unwrap_or_else(|_| {
        log::warn!("DATABASE_URL environment variable not set. Defaulting to 'form_data.db'.");
        "form_data.db".to_string()
    });
    // CRITICAL: Bind address is required
    let bind_address = env::var("BIND_ADDRESS").unwrap_or_else(|_| {
        log::warn!("BIND_ADDRESS environment variable not set. Defaulting to '127.0.0.1:8080'.");
        "127.0.0.1:8080".to_string()
    });
    // CRITICAL: Initial admin credentials (checked in db::init_db)
    // let initial_admin_username = env::var("INITIAL_ADMIN_USERNAME").expect("Missing INITIAL_ADMIN_USERNAME");
    // let initial_admin_password = env::var("INITIAL_ADMIN_PASSWORD").expect("Missing INITIAL_ADMIN_PASSWORD");
    // OPTIONAL: Allowed origin for CORS
    let allowed_origin = env::var("ALLOWED_ORIGIN").ok(); // Use ok() to make it optional

    log::info!(" --- Formies Backend Configuration ---");
    log::info!("Required Environment Variables:");
    log::info!(" - DATABASE_URL (Current: {})", database_url);
    log::info!(" - BIND_ADDRESS (Current: {})", bind_address);
    log::info!(" - INITIAL_ADMIN_USERNAME (Set during startup, MUST be present)");
    log::info!(" - INITIAL_ADMIN_PASSWORD (Set during startup, MUST be present)");
    log::info!("Optional Environment Variables:");
    if let Some(ref origin) = allowed_origin {
        log::info!(" - ALLOWED_ORIGIN (Set: {})", origin);
    } else {
        log::warn!(" - ALLOWED_ORIGIN (Not Set): CORS will be restrictive, potentially blocking browser access. Set to your frontend URL (e.g., http://localhost:5173 or https://yourdomain.com).");
    }
    log::info!(" - RUST_LOG (e.g., 'info,formies_be=debug')");
    log::info!(" --- End Configuration ---");

    // Initialize database connection
    let db_connection = match db::init_db(&database_url) {
        Ok(conn) => conn,
        Err(e) => {
            // Specific check for missing admin credentials error
            if e.to_string().contains("INITIAL_ADMIN_USERNAME")
                || e.to_string().contains("INITIAL_ADMIN_PASSWORD")
            {
                log::error!("FATAL: {}", e);
                log::error!("Please set the INITIAL_ADMIN_USERNAME and INITIAL_ADMIN_PASSWORD environment variables.");
            } else {
                log::error!(
                    "FATAL: Failed to initialize database at {}: {:?}",
                    database_url,
                    e
                );
            }
            process::exit(1); // Exit if DB initialization fails
        }
    };

    // Wrap connection in Arc<Mutex<>> for thread-safe sharing
    let db_data = web::Data::new(Arc::new(Mutex::new(db_connection)));

    log::info!("Starting server at http://{}", bind_address);

    HttpServer::new(move || {
        // Clone shared state for the closure
        let db_data_clone = db_data.clone();
        let allowed_origin_clone = allowed_origin.clone();

        // Configure CORS
        let cors = match allowed_origin_clone {
            Some(origin) => {
                log::info!("Configuring CORS for specific origin: {}", origin);
                Cors::default()
                    .allowed_origin(&origin) // Allow only the specified origin
                    .allowed_methods(vec!["GET", "POST", "PUT", "DELETE", "OPTIONS"])
                    .allowed_headers(vec![
                        header::AUTHORIZATION,
                        header::ACCEPT,
                        header::CONTENT_TYPE,
                        header::ORIGIN, // Add Origin header if needed
                        header::ACCESS_CONTROL_REQUEST_METHOD,
                        header::ACCESS_CONTROL_REQUEST_HEADERS,
                    ])
                    .supports_credentials()
                    .max_age(3600)
            }
            None => {
                // Default restrictive CORS: No origin allowed explicitly.
                // This will likely block browser requests unless the browser and server are on the same origin.
                log::warn!("CORS is configured restrictively: No ALLOWED_ORIGIN set.");
                Cors::default() // No allowed_origin set
                    .allowed_methods(vec!["GET", "POST", "PUT", "DELETE", "OPTIONS"])
                     .allowed_headers(vec![
                        header::AUTHORIZATION,
                        header::ACCEPT,
                        header::CONTENT_TYPE,
                        header::ORIGIN,
                        header::ACCESS_CONTROL_REQUEST_METHOD,
                        header::ACCESS_CONTROL_REQUEST_HEADERS,
                    ])
                    .supports_credentials()
                    .max_age(3600)
                // DO NOT use allow_any_origin() unless you fully understand the security implications.
            }
        };

        App::new()
            .wrap(cors) // Apply CORS middleware
            .wrap(Logger::default()) // Add request logging (default format)
            .app_data(db_data_clone) // Share database connection pool
            // --- API Routes ---
            .service(
                web::scope("/api") // Group API routes under /api
                    // --- Public Routes ---
                    .route("/login", web::post().to(handlers::login))
                    .route(
                        "/forms/{form_id}/submissions",
                        web::post().to(handlers::submit_form),
                    )
                    // --- Protected Routes (using Auth extractor) ---
                    .route("/logout", web::post().to(handlers::logout)) // Added logout
                    .route("/forms", web::post().to(handlers::create_form))
                    .route("/forms", web::get().to(handlers::get_forms))
                    .route(
                        "/forms/{form_id}/submissions",
                        web::get().to(handlers::get_submissions),
                    ),
            )
            // --- Static Files (Serve Frontend - Optional) ---
            // Assumes frontend build output is in ../frontend/dist
            // Register this LAST to avoid conflicts with API routes
            .service(
                fs::Files::new("/", "../frontend/dist/")
                    .index_file("index.html")
                    .use_last_modified(true)
                    // Optional: Add a fallback to index.html for SPA routing
                    .default_handler(
                        fs::NamedFile::open("../frontend/dist/index.html").unwrap_or_else(|_| {
                            log::error!("Fallback file not found: ../frontend/dist/index.html");
                            process::exit(1); // Exit if fallback file is missing
                        }), // Handle error explicitly
                    ),
            )
    })
    .bind(&bind_address)?
    .run()
    .await
}
