// src/models.rs
use serde::{Deserialize, Serialize};
// Consider adding chrono for DateTime types if needed in responses
// use chrono::{DateTime, Utc};

// Represents the structure for defining a form
#[derive(Serialize, Deserialize, Debug, Clone)]
pub struct Form {
    #[serde(skip_serializing_if = "Option::is_none")]
    pub id: Option<String>,
    pub name: String,
    /// Stores the structure defining the form fields.
    /// Expected to be a JSON array of field definition objects.
    /// Example field definition object:
    /// ```json
    /// {
    ///   "name": "email",                  // String, required: Unique identifier for the field
    ///   "type": "email",                  // String, required: "string", "number", "boolean", "email", "url", "object", "array"
    ///   "label": "Email Address",         // String, optional: User-friendly label
    ///   "required": true,                 // Boolean, optional (default: false): If the field must have a value
    ///   "placeholder": "you@example.com", // String, optional: Placeholder text
    ///   "minLength": 5,                   // Number, optional: Minimum length for strings
    ///   "maxLength": 100,                 // Number, optional: Maximum length for strings
    ///   "min": 0,                         // Number, optional: Minimum value for numbers
    ///   "max": 100,                       // Number, optional: Maximum value for numbers
    ///   "pattern": "^\\S+@\\S+\\.\\S+$"     // String, optional: Regex pattern for strings (e.g., email, url types might use this implicitly or explicitly)
    ///   // Add other properties like "options" for select/radio, etc.
    /// }
    /// ```
    pub fields: serde_json::Value,
    // Optional: Add created_at if needed in API responses
    // pub created_at: Option<DateTime<Utc>>,
}

// Represents a single submission for a specific form
#[derive(Serialize, Deserialize, Debug, Clone)]
pub struct Submission {
    pub id: String,
    pub form_id: String,
    /// Stores the data submitted by the user.
    /// Expected to be a JSON object where keys are field names (`name`) from the form definition's `fields` array.
    /// Example: `{ "email": "user@example.com", "age": 30 }`
    pub data: serde_json::Value,
    // Optional: Add created_at if needed in API responses
    // pub created_at: Option<DateTime<Utc>>,
}

// Used for the /login endpoint request body
#[derive(Debug, Serialize, Deserialize)]
pub struct LoginCredentials {
    pub username: String,
    pub password: String,
}

// Used for the /login endpoint response body
#[derive(Debug, Serialize, Deserialize)]
pub struct LoginResponse {
    pub token: String, // The session token (UUID)
}

// Used internally to represent a user fetched from the DB for authentication check
// Not serialized, only used within db.rs and handlers.rs
#[derive(Debug)]
pub struct UserAuthData {
    pub id: String,
    pub hashed_password: String,
    // Note: Token and expiry are handled separately and not needed in this specific struct
}

// --- Custom Application Error (Optional but Recommended for Consistency) ---
// Although not fully integrated in this pass to minimize changes,
// this shows the structure for future improvement.

// use actix_web::{ResponseError, http::StatusCode};
// use std::fmt;

// #[derive(Debug)]
// pub enum AppError {
//     DatabaseError(anyhow::Error),
//     ConfigError(String),
//     ValidationError(serde_json::Value), // Store the validation errors JSON
//     NotFound(String),
//     Unauthorized(String),
//     InternalError(String),
//     BlockingError(String),
// }

// impl fmt::Display for AppError {
//     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
//         match self {
//             AppError::DatabaseError(e) => write!(f, "Database error: {}", e),
//             AppError::ConfigError(s) => write!(f, "Configuration error: {}", s),
//             AppError::ValidationError(_) => write!(f, "Validation failed"),
//             AppError::NotFound(s) => write!(f, "Not found: {}", s),
//             AppError::Unauthorized(s) => write!(f, "Unauthorized: {}", s),
//             AppError::InternalError(s) => write!(f, "Internal server error: {}", s),
//             AppError::BlockingError(s) => write!(f, "Blocking operation error: {}", s),
//         }
//     }
// }

// impl ResponseError for AppError {
//     fn status_code(&self) -> StatusCode {
//         match self {
//             AppError::DatabaseError(_) => StatusCode::INTERNAL_SERVER_ERROR,
//             AppError::ConfigError(_) => StatusCode::INTERNAL_SERVER_ERROR,
//             AppError::ValidationError(_) => StatusCode::BAD_REQUEST,
//             AppError::NotFound(_) => StatusCode::NOT_FOUND,
//             AppError::Unauthorized(_) => StatusCode::UNAUTHORIZED,
//             AppError::InternalError(_) => StatusCode::INTERNAL_SERVER_ERROR,
//             AppError::BlockingError(_) => StatusCode::INTERNAL_SERVER_ERROR,
//         }
//     }

//     fn error_response(&self) -> HttpResponse {
//         let status = self.status_code();
//         let error_json = match self {
//             AppError::ValidationError(errors) => errors.clone(),
//              // Provide a generic error structure for others
//             _ => json!({ "error": status.canonical_reason().unwrap_or("Unknown Error"), "message": self.to_string() }),
//         };

//         HttpResponse::build(status).json(error_json)
//     }
// }

// // Implement From traits to convert other errors into AppError easily
// impl From<anyhow::Error> for AppError {
//     fn from(err: anyhow::Error) -> Self {
//         // Basic conversion, could add more context analysis here
//         AppError::DatabaseError(err)
//     }
// }
// impl From<actix_web::error::BlockingError> for AppError {
//    fn from(err: actix_web::error::BlockingError) -> Self {
//        AppError::BlockingError(err.to_string())
//    }
//}
// // Add From<rusqlite::Error>, From<serde_json::Error>, etc. as needed
